
//globally referenceable sound process
//keeps servers updated.

EventDef {
	classvar <all, <>servers;
	var <>func;
	
	*new { arg key, func;
		var synthDef;
		if(func.isNil, { ^all.at(key.asSymbol) });
		^super.newCopyArgs(func).toLib(key.asSymbol)
	}

	toLib { arg key;
		this.class.all.put(key, this);
	}
	argNames { ^func.def.argNames }
	
	*at { arg key;
		^all.at(key)
	}
	valueArrayEnvir { arg args;
		^func.valueArrayEnvir(args)
	}

}



SoundDef : EventDef {
	var <synthDef, argNames;
	
	*new { arg key, func, rates, prependArgs;
		var synthDef;
		if(func.isNil, { ^all.at(key.asSymbol) });
		synthDef = SynthDef(key, func, rates, prependArgs);
		^super.new(key, func).init(synthDef) // revisit
	}
	
	*initClass { 
		Class.initClassTree(Server);
		all = IdentityDictionary.new;
		servers = [Server.default]
	}
	
	init { arg def;
		synthDef = def;
		this.send;
		argNames = synthDef.controlNames.collect({ arg item; item.name })
	}
	
	controlNames {^this.argNames }
		
	*sendAll {
		all.do({ arg item; item.send });
	}
	
	*loadAll {
		all.do({ arg item; item.load });
	}
	
	newMsg { arg args, target, addActionNumber=0, id;
		^[ 9, synthDef.name, id ? -1, addActionNumber, target.asTarget.nodeID ]++args
	}
	
	// change this: should be using PolyPlayer! (or pass player in?)
	valueArrayEnvir { arg args;
		var srv, id;
		srv = ~server;
		id = srv.nextNodeID;
		srv.sendBundle(srv.latency, this.newMsg(args, srv, 0, id));
		if(synthDef.canReleaseSynth) {
			~clock ? TempoClock.default.sched(~sustain + ~lag, {
				srv.sendBundle(srv.latency, ["/n_set", id, \gate, 0]); // check lag!
			})
		}
	}
	
	asDefName {
		^synthDef.name
	}
		
	send { 	
		this.class.servers.do({ arg server;
			synthDef.send(server);
		})
	}
	
	load { arg server;
		this.class.servers.do({ arg server;
			synthDef.load(server);
		})
	}
	
	play { arg args, target;
		^Synth.new(this, args, target)
	}
	
}